package zmostp_go

type DecodeFailureLogHandler interface {
	DFAInvalidState(st DFAState)
	DFAInvalidInput(st DFAState, b byte)
	DecodeChannelAscii85Failed(data []byte, err error)
	DecodePayloadAscii85Failed(ch uint32, data []byte, err error)
	DecodeChecksumAscii85Failed(ch uint32, data []byte, err error)
	NeedCRCChecksum()
	CRCChecksumMismached(crcFromData uint32, crcReceived uint32)
	InvalidChannelByteLength(data []byte)
	InvalidChecksumByteLength(ch uint32, data []byte)
}

type EmptyDecodeFailureLogHandler struct{}

func (e EmptyDecodeFailureLogHandler) InvalidChannelByteLength(data []byte) {
}

func (e EmptyDecodeFailureLogHandler) InvalidChecksumByteLength(ch uint32, data []byte) {
}

func (e EmptyDecodeFailureLogHandler) CRCChecksumMismached(want uint32, got uint32) {
}

func (e EmptyDecodeFailureLogHandler) DecodeChecksumAscii85Failed(ch uint32, data []byte, err error) {
}

func (e EmptyDecodeFailureLogHandler) NeedCRCChecksum() {
}

func (e EmptyDecodeFailureLogHandler) DFAInvalidState(st DFAState) {
}

func (e EmptyDecodeFailureLogHandler) DecodePayloadAscii85Failed(ch uint32, data []byte, err error) {
}

func (e EmptyDecodeFailureLogHandler) DecodeChannelAscii85Failed(data []byte, err error) {
}

func (e EmptyDecodeFailureLogHandler) DFAInvalidInput(st DFAState, b byte) {
}

var _ DecodeFailureLogHandler = (*EmptyDecodeFailureLogHandler)(nil)
